/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.registry.core;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.petals.registry.api.Registry;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.lifecycle.LifeCycle;
import org.ow2.petals.registry.api.util.Assert;

/**
 * This class is used to handle resources and lifecycle. A resource can only be
 * handled when the lifecycle is in started state.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public abstract class AbstractRegistry implements Registry {

    protected LifeCycle lifeCycle;

    protected String name;

    protected Context context;

    protected Log logger = LogFactory.getLog(AbstractRegistry.class);

    /**
     * Creates a new instance of AbstractRegistry
     */
    public AbstractRegistry() {
    }

    public final boolean delete(String key, boolean b) throws RegistryException {
        if (this.isStarted()) {
            return this.doDelete(key, b);
        } else {
            throw new LifeCycleException("Bad state " + this.getState());
        }
    }

    /**
     * 
     * @param uuid
     * @throws RegistryException
     */
    public abstract boolean doDelete(String key, boolean b) throws RegistryException;

    public final Resource get(String key, boolean b) throws RegistryException {
        if (this.isStarted()) {
            return this.doGet(key, b);
        } else {
            throw new RegistryException("Invalid state " + this.getState());
        }
    }

    public final List<Resource> getAll(String path, boolean b) throws RegistryException {
        if (this.isStarted()) {
            return this.doGetAll(path, b);
        } else {
            throw new RegistryException("Invalid state " + this.getState());
        }
    }

    public void setName(String name) {
        this.name = name;
    }

    public abstract Resource doGet(String key, boolean b) throws RegistryException;

    public abstract List<Resource> doGetAll(String key, boolean b) throws RegistryException;

    public final boolean put(String key, Resource resource, boolean b) throws RegistryException {
        boolean result = false;
        if (this.isStarted()) {
            resource.setRootRegistry(this.getName());
            result = this.doPut(key, resource, b);
        } else {
            throw new RegistryException("Invalid state " + this.getState());
        }
        return result;
    }

    /**
     * 
     * @param resource
     * @return
     */
    public abstract boolean doPut(String key, Resource resource, boolean b)
            throws RegistryException;

    /**
     * 
     * @param lifeCycle
     */
    protected void setLifeCycle(LifeCycle lifeCycle) {
        this.lifeCycle = lifeCycle;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#getState()
     */
    public STATE getState() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.getState();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#init()
     */
    public void init(Context context) throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        Assert.notNull(context, "Context can not be null");

        if (this.logger.isDebugEnabled()) {
            this.logger.debug("AbstractRegistry : init()");
        }

        this.context = context;
        this.context.init();
        this.lifeCycle.init(this.context);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isInitialized()
     */
    public boolean isInitialized() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isInitialized();
    }

    public boolean isNotInitialized() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isNotInitialized();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isRunning()
     */
    public boolean isRunning() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isRunning();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isStarted()
     */
    public boolean isStarted() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isStarted();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isStopped()
     */
    public boolean isStopped() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isStopped();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#start()
     */
    public void start() throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.start();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#stop()
     */
    public void stop() throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.stop();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.CoreRegistry#getName()
     */
    public String getName() {
        return this.name;
    }

    /**
     * @return the context
     */
    public Context getContext() {
        return this.context;
    }

    public void setContext(Context context) {
        this.context = context;
    }
}
