/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.registry.core.manager;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.petals.registry.api.Information;
import org.ow2.petals.registry.api.LocalRegistry;
import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.ReceptionException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.lifecycle.AbstractLifeCycle;
import org.ow2.petals.registry.api.manager.IncomingManager;
import org.ow2.petals.registry.api.util.Assert;

/**
 * An abstract local manager is used to handle incoming messages from remote
 * registries.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class GenericIncomingManagerImpl implements IncomingManager {

    protected AbstractLifeCycle lifeCycle;

    protected LocalRegistry localRegistry;

    protected String name;

    protected boolean receive;

    private final Log log = LogFactory.getLog(GenericIncomingManagerImpl.class);

    /**
     * Creates a new instance of AbstractIncomingManager
     * 
     * @param incomingMessageDispatcher
     */
    public GenericIncomingManagerImpl() {
        super();
        this.receive = true;
        this.lifeCycle = new AbstractLifeCycle() {
            @Override
            protected void doInit(Context context) throws LifeCycleException {
                if (GenericIncomingManagerImpl.this.getLocalRegistry() == null) {
                    throw new LifeCycleException("Local registry has not been set");
                }
            }
        };
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.CoreRegistry#getName()
     */
    public String getName() {
        return this.name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.CoreRegistry#setName(java.lang.String)
     */
    public void setName(String name) {
        this.name = name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LocalManager#getLocalRegistry()
     */
    public LocalRegistry getLocalRegistry() {
        return this.localRegistry;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.registry.core.LocalManager#setLocalRegistry(org.ow2.petals
     * .registry.core.LocalRegistry)
     */
    public void setLocalRegistry(LocalRegistry localRegistry) throws LifeCycleException {
        if (!this.isNotInitialized()) {
            throw new LifeCycleException("Can not set the local registry in the state "
                    + this.getState());
        }
        this.localRegistry = localRegistry;
    }

    public void pause() {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Suspending message reception");
        }
        this.receive = false;
    }

    public void resume() {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Resuming message reception");
        }
        this.receive = true;
    }

    protected void checkReceive() throws ReceptionException {
        if (!this.receive) {
            throw new ReceptionException("Receive data is not started or suspended");
        }
    }

    protected void checkPath(String path) throws RegistryException {
        if (path == null) {
            throw new RegistryException("Path is null");
        }
    }

    /**
     * Check that the resource can be received by the current registry. A basic
     * rule is that a resource created on the local registry can not be received
     * by the same...
     * 
     * @param resource
     */
    protected void checkSource(Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Check that the operation information is OK");
        }

        if (information == null) {
            throw new RegistryException("No information means no access!!!");
        }

        if (information.getSender().equals(this.localRegistry.getName())) {
            throw new RegistryException("Can not receive a locally created resource");
        }
    }

    /*
     * TODO : Things to be added on the data control (revision etc...) on the
     * operations implementations here not on the user implementation...
     */
    public final Resource get(String path, Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Get the resource " + path);
        }

        this.checkSource(information);
        this.checkPath(path);
        this.checkReceive();

        return this.doGet(path);
    }

    protected Resource doGet(String path) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Got a get call");
        }

        if (path == null) {
            throw new RegistryException("Can not get a resource with key = null");
        }

        return this.getLocalRegistry().getRepository().get(path, false);
    }

    public final boolean put(String path, Resource resource, Information information)
            throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Put the resource " + path);
        }

        this.checkSource(information);
        this.checkPath(path);
        this.checkReceive();
        return this.doPut(path, resource);
    }

    protected boolean doPut(String key, Resource resource) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Put the resource " + resource);
        }

        if (resource == null) {
            throw new RegistryException("Can not put a null resource");
        }

        return this.getLocalRegistry().getRepository().put(key, resource, false);
    }

    public final boolean delete(String path, Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Delete the resource " + path);
        }

        this.checkSource(information);
        this.checkPath(path);
        this.checkReceive();
        return this.doDelete(path);
    }
    
    protected boolean doDelete(String key) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Got a delete request");
        }

        if (key == null) {
            throw new RegistryException("Can not delete a resource with key = null");
        }

        // just delete the resource, do not call the registry since it will
        // propagate the call to all the peers...
        this.getLocalRegistry().getRepository().delete(key, false);

        return true;
    }

    public final List<Resource> getAll(String path, Information information)
            throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Get all the resources from " + path);
        }

        this.checkSource(information);
        this.checkPath(path);
        this.checkReceive();
        return this.doGetAll(path);
    }

    protected List<Resource> doGetAll(String path) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Got a getall call");
        }

        if (path == null) {
            throw new RegistryException("Can not get a resource with key = null");
        }

        return this.getLocalRegistry().getRepository().getAll(path, false);
    }

    public final List<Resource> query(Query query, Information information)
            throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Query with " + query);
        }

        this.checkSource(information);
        this.checkReceive();
        return this.doQuery(query);
    }

    protected List<Resource> doQuery(Query query) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Got a query call " + query);
        }

        if (query == null) {
            throw new RegistryException("Can not query with a null query");
        }

        return this.getLocalRegistry().getRepository().query(query, false);
    }

    /**
     * 
     * @param lifeCycle
     */
    protected void setLifeCycle(AbstractLifeCycle lifeCycle) {
        this.lifeCycle = lifeCycle;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#getState()
     */
    public STATE getState() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.getState();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#init()
     */
    public void init(Context context) throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.init(context);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isInitialized()
     */
    public boolean isInitialized() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isInitialized();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isNotInitialized()
     */
    public boolean isNotInitialized() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isNotInitialized();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isRunning()
     */
    public boolean isRunning() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isRunning();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isStarted()
     */
    public boolean isStarted() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isStarted();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isStopped()
     */
    public boolean isStopped() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isStopped();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#start()
     */
    public void start() throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.start();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#stop()
     */
    public void stop() throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.stop();
    }
}
