/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.registry.core.repository;

import java.util.List;

import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.context.Workspace;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.exception.RepositoryException;
import org.ow2.petals.registry.api.lifecycle.LifeCycle;
import org.ow2.petals.registry.api.repository.Repository;
import org.ow2.petals.registry.api.util.Assert;
import org.ow2.petals.registry.api.util.ResourceUtil;

/**
 * An abstract respository definition. This abstract class handles the requests
 * from the registry and use the transaction manager in order to get a coherent
 * set of data. For example, working with resource in the DAO is not only a
 * unique SQL query... All the repository operations are waiting to get a
 * connection from the transaction manager.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public abstract class AbstractRepository implements Repository {

    protected LifeCycle lifeCycle;

    protected String name;

    protected Workspace workspace;

    protected Context context;

    /**
     * Creates a new instance of AbstractRepository
     */
    public AbstractRepository() {
        super();
        this.createLifeCycle();
    }

    /**
     * 
     * @return
     */
    public void createLifeCycle() {
        if (this.lifeCycle == null) {
            this.lifeCycle = new RepositoryLifecycle(this);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.Registry#get(java.util.UUID)
     */
    public final Resource get(String key, boolean b) throws RegistryException {
        if (!this.isStarted()) {
            throw new LifeCycleException("Can not get a resource when repository is in state "
                    + this.getState());
        }

        if (ResourceUtil.isDirectory(key)) {
            throw new RepositoryException("Can not get a single resource from a directory path");
        }

        Resource r = null;
        boolean success = false;

        try {
            // this.beginTransaction();
            r = this.doGet(key);
            success = true;
        } catch (RegistryException e) {
            success = false;
            throw e;
        } finally {
            if (success) {
                // this.commitTransaction();
            } else {
                // this.rollbackTransaction();
            }
        }
        return r;
    }

    public List<Resource> getAll(String path, boolean b) throws RegistryException {
        if (!this.isStarted()) {
            throw new LifeCycleException("Can not get a resource when repository is in state "
                    + this.getState());
        }

        if (!ResourceUtil.isDirectory(path)) {
            throw new RepositoryException(
                    "Can not get a directory listing from a single resource path");
        }

        List<Resource> r = null;
        boolean success = false;

        try {
            // this.beginTransaction();
            r = this.doGetAll(path);
            success = true;
        } catch (RegistryException e) {
            success = false;
            throw e;
        } finally {
            if (success) {
                // this.commitTransaction();
            } else {
                // this.rollbackTransaction();
            }
        }
        return r;
    }

    public synchronized final boolean put(String path, Resource resource, boolean b)
            throws RegistryException {
        if (!this.isStarted()) {
            throw new LifeCycleException("Can not put a resource when repository is in state "
                    + this.getState());
        }

        if (path == null) {
            throw new RepositoryException("Can not put the resource under a null path...");
        }

        if (ResourceUtil.isDirectory(path)) {
            throw new RepositoryException("Can not put the resource under a directory path '"
                    + path + "'");
        }

        // put the path into the resource now...
        if ((resource != null) && !path.equals(resource.getPath())) {
            resource.setPath(path);
        }

        boolean success = false;
        try {
            this.doPut(path, resource);
            success = true;
        } catch (RegistryException e) {
            success = false;
            throw e;
        } finally {
        }
        return success;
    }

    public final boolean delete(String key, boolean b) throws RegistryException {
        if (!this.isStarted()) {
            throw new LifeCycleException("Can not delete a resource when repository is in state "
                    + this.getState());
        }

        boolean success = false;
        try {
            this.doDelete(key);
            success = true;
        } catch (RegistryException e) {
            success = false;
            throw e;
        } finally {
        }
        return success;
    }

    public final int clean() throws RegistryException {
        int result = 0;
        if (!this.isStarted()) {
            throw new LifeCycleException(
                    "Can not clean the repository when the repository is in state "
                            + this.getState());
        }

        try {
            result = this.doClean();
        } catch (RegistryException e) {
            throw e;
        } finally {
        }
        return result;
    }

    public final long size() throws RegistryException {
        long result = 0L;
        if (!this.isStarted()) {
            throw new LifeCycleException(
                    "Can not get the size of the repository when the repository is in state "
                            + this.getState());
        }

        try {
            result = this.doSize();
        } catch (RegistryException e) {
            throw e;
        } finally {
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public int keepOnly(String registryName) throws RegistryException {
        int result = 0;
        if (!this.isStarted()) {
            throw new LifeCycleException("Can not keep only data for registry " + registryName
                    + " in state " + this.getState());
        }

        try {
            result = this.doKeepOnly(registryName);
        } catch (RegistryException e) {
            throw e;
        } finally {
        }
        return result;
    }

    protected abstract int doKeepOnly(String registryName) throws RegistryException;

    /**
     * {@inheritDoc}
     */
    public int deleteDataFromRegistry(String registryName) throws RegistryException {
        int result = 0;
        if (!this.isStarted()) {
            throw new LifeCycleException(
                    "Can not delete data from other registries in the repository when the repository is in state "
                            + this.getState());
        }

        try {
            result = this.doDeleteDataFromRegistry(registryName);
        } catch (RegistryException e) {
            throw e;
        } finally {
        }
        return result;
    }

    protected abstract int doDeleteDataFromRegistry(String registryName) throws RegistryException;

    public List<Resource> query(Query query, boolean b) throws RegistryException {
        List<Resource> result = null;
        if (!this.isStarted()) {
            throw new LifeCycleException(
                    "Can not clean the repository when the repository is in state "
                            + this.getState());
        }

        try {
            result = this.doQuery(query);
        } catch (RegistryException e) {
            throw e;
        } finally {
        }
        return result;
    }

    protected abstract Resource doGet(String path) throws RegistryException;

    protected abstract List<Resource> doGetAll(String path) throws RegistryException;

    protected abstract boolean doPut(String path, Resource resource) throws RegistryException;

    protected abstract boolean doDelete(String path) throws RegistryException;

    protected abstract int doClean() throws RegistryException;

    protected abstract long doSize() throws RegistryException;

    protected abstract List<Resource> doQuery(Query query) throws RegistryException;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.repository.Repository#getContext()
     */
    public Context getContext() {
        return this.context;
    }

    /**
     * 
     * @param lifeCycle
     */
    protected void setLifeCycle(LifeCycle lifeCycle) {
        this.lifeCycle = lifeCycle;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#getState()
     */
    public STATE getState() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.getState();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#init()
     */
    public void init(Context context) throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        Assert.notNull(context);

        // the repository loads the context since it is linked to the workspace
        // initialisation
        this.context = context;
        this.context.init();
        this.workspace = context.getWorkspace();
        this.lifeCycle.init(context);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isInitialized()
     */
    public boolean isInitialized() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isInitialized();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isNotInitialized()
     */
    public boolean isNotInitialized() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isNotInitialized();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isRunning()
     */
    public boolean isRunning() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isRunning();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isStarted()
     */
    public boolean isStarted() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isStarted();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#isStopped()
     */
    public boolean isStopped() {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        return this.lifeCycle.isStopped();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#start()
     */
    public void start() throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.start();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.registry.core.LifeCycle#stop()
     */
    public void stop() throws LifeCycleException {
        Assert.notNull(this.lifeCycle, "No lifecycle");
        this.lifeCycle.stop();
    }
}
